@@ -3,7 +3,12 @@ from functools import partial, reduce
 
 
 def unique(ll: List) -> List:
-    """Return unique elements in a list preserving order."""
+    """Return unique elements in a list preserving order.
+
+    Args:
+        ll: The list on which to operate
+
+    """
     ret = []
     for x in ll:
         if x not in ret:
@@ -17,12 +22,14 @@ def identity(x: Any):
     Example:
         >>> identity(10)
         10
-        >>> identity(None)
-        None
+        >>> identity(None)        # Returns None
     """
     return x
 
 
+# FIXME: rpartial is buggy and the last example here fails
+#        as keywords can cause conflicts with arg applications
+#        We get "got two values for 'b'" in the last example
 def rpartial(func: Callable, *args, **kwargs):
     """Partial application of function `func` in reverse.
 
@@ -33,14 +40,14 @@ def rpartial(func: Callable, *args, **kwargs):
 
     Example:
         >>> def func(a, b, c,):
-        >>>     print(a, b, c)
+        ...     print(a, b, c)
 
         # b and c take the place of 'b' and 'c' in the function arguments here
         # That is, left to right
         >>> rpartial(func, 2, 3)(1)
-        1, 2, 3
+        1 2 3
 
-        >>> rpartial(func, 3, b=2)
+        >>> rpartial(func, 3, b=2)(1)
 
     """
     def temp(*rest):
@@ -53,7 +60,7 @@ def maybe_is(x: Any, kinds: List[type]) -> Iterable[bool]:
 
     Example:
         >>> var = "var"
-        >>> maybe_is(var, [bool, str, type])
+        >>> [*maybe_is(var, [bool, str, type])]
         [False, True, False]
     """
     return map(partial(apply, isinstance), [(x, k) for k in kinds])
@@ -68,10 +75,10 @@ def maybe_(x: Any, kinds: List[type]) -> type:
     Example:
         >>> var = "var"
         >>> maybe_(var, [bool, str, type])
-        str
+        <class 'str'>
 
         >>> maybe_(str, [bool, str, type])
-        type
+        <class 'type'>
 
     """
     return last_item(first_by(zip(maybe_is(x, kinds), kinds), car))
@@ -88,7 +95,7 @@ def maybe_then(x: Any, kinds: List[type], then: List[Callable]) -> Any:
         3
 
         >>> class Var:
-        >>>     pass
+        ...     pass
         >>> var = Var()
         >>> maybe_then(Var, kinds, funcs)
         'Var'
@@ -101,12 +108,34 @@ def maybe_then(x: Any, kinds: List[type], then: List[Callable]) -> Any:
 
 
 def foldl(func: Callable, struct: Iterable):
+    """Fold from left
+
+    Args:
+        func: Function to call on each argument
+        struct: Structure on which to fold
+
+    """
+    it = iter(struct)
+    yield(func(next(it)))
+
+
+def foldr(func: Callable, struct: Iterable):
+    """Fold from right.
+
+    Actually :func:`foldr` cannot really be well implemented in python unless
+    the structure is reversible in constant time or we have a `yieldr` function.
+
+    Args:
+        func: Function to call on each argument
+        struct: Structure on which to fold
+
+    """
     it = iter(struct)
     yield(func(next(it)))
 
 
 def first_by(struct: Optional[Iterable], by: Callable, predicate: Callable = identity) -> Any:
-    """Return first item in list on which `predicate` on output of `by` is True.
+    """Return first item in list on which :code:`predicate` on output of :code:`by` is True.
 
     Return:
         Either the found item or None
@@ -124,12 +153,12 @@ def first_by(struct: Optional[Iterable], by: Callable, predicate: Callable = ide
 
 def any_attr(obj: object, attrs: List[str],
              predicate: Callable[[Any], bool] = identity):
-    """Return True if any attribute in `obj` satisfies `predicate` for a given
-    list of attributes `attrs`.
+    """Return True if any attribute in :code:`obj` satisfies :code:`predicate` for a given
+    list of attributes :code:`attrs`.
 
     Args:
-        obj: Any `python` object
-        attrs: A `list` of names of attributes
+        obj: Any :code:`python` object
+        attrs: A :code:`list` of names of attributes
         predicat: A boolean function
 
     """
@@ -138,12 +167,12 @@ def any_attr(obj: object, attrs: List[str],
 
 def all_attrs(obj: object, attrs: List[str],
               predicate: Callable[[Any], bool] = identity):
-    """Return True if all attributes in `obj` satisfy `predicate` for a given
-    list of attributes `attrs`.
+    """Return True if all attributes in :code:`obj` satisfy :code:`predicate` for a given
+    list of attributes :code:`attrs`.
 
     Args:
-        obj: Any `python` object
-        attrs: A `list` of names of attributes
+        obj: Any :code:`python` object
+        attrs: A :code:`list` of names of attributes
         predicat: A boolean function
 
     """
@@ -152,6 +181,13 @@ def all_attrs(obj: object, attrs: List[str],
 
 def last_item(struct: Optional[Iterable]):
     """Return last item of `struct`.
+
+    Args:
+        struct: The struct of which to get the last item
+
+    Again this isn't very practical for python or in fact, for singly linked
+    lists in general.
+
     """
     if struct:
         it = iter(struct)
@@ -189,6 +225,12 @@ def first_item(struct: Iterable):
 
 def nth(struct: Iterable, indx: int):
     """Return nth item of `struct`.
+
+    Example:
+        >>> d = {x: x for x in range(10)}
+        >>> nth(d, 1)
+        1
+
     """
     it = iter(struct)
     i = 0
@@ -203,7 +245,7 @@ def nth(struct: Iterable, indx: int):
 
 
 def applify(func: Callable, struct: Iterable[Iterable]):
-    """`` all the iters and apply function `func` to each of them
+    """:func:`zip` all the iters and apply function `func` to each of them
     """
     return map(partial(apply, func), struct)
 
@@ -231,7 +273,7 @@ def apply(func: Callable, args: List):
 
     Example:
         >>> def add(a, b):
-        >>>     return a + b
+        ...     return a + b
         >>> apply(add, [1, 2])
         3
     """
@@ -249,14 +291,14 @@ def pipe(*args: Callable) -> Callable:
 
     Example:
         >>> def f(x: int):
-        >>>     print(x)
-        >>>     return str(x)
-        >>>
+        ...     print(x)
+        ...     return str(x)
+
         >>> def g(x: str):
-        >>>     return "func g " + x
-        >>>
+        ...     return "func g " + x
+
         >>> def h(x: str):
-        >>>     return "func h " + x
+        ...     return "func h " + x
 
         >>> pipe(f, g, h)(10)
         10
@@ -278,23 +320,27 @@ def thunkify(*args: Callable) -> Callable:
     evaluation of arguments. Can be used for side effects.
 
     Example:
-        def f(x: int):
-            print("func f", x)
+        >>> def f(x: int):
+        ...     print("func f", x)
 
-        def g():
-            print("func g")
+        >>> def g():
+        ...     print("func g")
 
-        val = 10
-        thunk = thunkify(partial(f, val), g)
-        thunk()   # prints "func f 10" and "func g"
+        >>> val = 10
+        >>> thunk = thunkify(partial(f, val), g)
+        >>> thunk()
+        func f 10
+        func g
 
         # Or if val won't be available till later
-        thunk = thunkify(f, lambda *_: g())
-        some_other_func(arg1, arg2, thunk)
+        >>> thunk = thunkify(f, lambda *_: g())
+        >>> some_other_func(arg1, arg2, thunk)
 
-        # In some_other_func
-        val = 20
-        thunk(val)   # prints "func f 20" and "func g"
+        >>> # In some_other_func
+        >>> val = 20
+        >>> thunk(val)   # prints "func f 20" and "func g"
+        func f 20
+        func g
 
     """
     def thunk(*_args):
@@ -351,30 +397,53 @@ def lens(obj: Optional[Dict], *args) -> Any:
 
 
 def difference(a: Iterable, b: Iterable) -> set:
-    """Return :class:`set` difference of two iterables"""
+    """Return :class:`set` difference of two iterables
+
+    Args:
+        a: first iterable
+        b: second iterable
+
+    """
     a = set([*a])
     b = set([*b])
     return a - b
 
 
 def intersection(a: Iterable, b: Iterable) -> set:
-    """Return :class:`set` intersection of two iterables"""
+    """Return :class:`set` intersection of two iterables
+
+    Args:
+        a: first iterable
+        b: second iterable
+
+    """
     a = set([*a])
     b = set([*b])
     return a.intersection(b)
 
 
 def union(a: Iterable, b: Iterable) -> set:
-    """Return :class:`set` union of two iterables"""
+    """Return :class:`set` union of two iterables
+
+    Args:
+        a: first set
+        b: second set
+
+    """
     a = set([*a])
     b = set([*b])
     return a.union(b)
 
 
-def concat(list_var: Iterable[List]) -> List:
-    """Concat all items in a given list of lists"""
+def concat(_list: Iterable[List]) -> List:
+    """Return a new list from concatenating a given list of lists
+
+    Args:
+        _list: The list of lists
+
+    """
     temp = []
-    for x in list_var:
+    for x in _list:
         temp.extend(x)
     return temp
 
@@ -382,18 +451,41 @@ def concat(list_var: Iterable[List]) -> List:
 def takewhile(predicate: Callable[[Any], bool], seq: Iterable) -> Iterable:
     """Lazily evaluated takewhile
 
-    :param predicate: First failure of predicate stops the iteration. Should return bool
-    :param seq: Sequence from which to take
-    :returns: filtered sequence
-    :rtype: Same as `seq`
+    Args:
+        predicate: First failure of predicate stops the iteration. Should return bool
+        seq: Sequence from which to take
 
     """
     it = iter(seq)
-    _next = it.__next__()
-    while predicate(_next):
-        try:
+    try:
+        _next = it.__next__()
+        while predicate(_next):
             yield _next
             _next = it.__next__()
+    except StopIteration:
+        return None
+
+
+def dropwhile(predicate: Callable[[Any], bool], seq: Iterable) -> Iterable:
+    """Lazily evaluated dropwhile.
+
+    Args:
+        predicate: A `Callable` that returns a bool.
+                   The iterable after first success of predicate is returned.
+        seq: Sequence from which to drop
+
+    """
+    it = iter(seq)
+    try:
+        _next = it.__next__()
+        while predicate(_next):
+            _next = it.__next__()
+    except StopIteration:
+        return None
+    while(_next):
+        yield _next
+        try:
+            _next = it.__next__()
         except StopIteration:
             return None
 
@@ -431,18 +523,18 @@ def flatten(_list: List, depth: Optional[int] = None):
     If depth is not given then the list is flattened as much as possible.
 
     Example:
-        flatten([[0, 1], [2, 3]])
-        >>> [0, 1, 2, 3]
+        >>> flatten([[0, 1], [2, 3]])
+        [0, 1, 2, 3]
 
-        flatten([[0, 1], [2, [3, 4]]])
-        >>> [0, 1, 2, 3, 4]
+        >>> flatten([[0, 1], [2, [3, 4]]])
+        [0, 1, 2, 3, 4]
 
-        flatten([[0, 1], [2, [3, 4]]], 1)
-        >>> [0, 1, 2, [3, 4]]
+        >>> flatten([[0, 1], [2, [3, 4]]], 1)
+        [0, 1, 2, [3, 4]]
 
         # Will not flatten a set
-        flatten([[0, 1], [2, {3, 4}]])
-        >>> [0, 1, 2, {3, 4}]
+        >>> flatten([[0, 1], [2, {3, 4}]])
+        [0, 1, 2, {3, 4}]
     """
     retval = []
     if depth is not None:
@@ -467,32 +559,53 @@ def map_if(func: Callable, pred: Callable[..., bool], struct: Iterable) -> list:
     """
     retval = []
     it = iter(struct)
-    _next = it.__next__()
-    while _next:
-        try:
+    try:
+        _next = it.__next__()
+        while _next:
             retval.append(func(_next) if pred(_next) else _next)
             _next = it.__next__()
-        except StopIteration:
-            return retval
+    except StopIteration:
+        return retval
     return retval
 
 
-def exactly_one(*_args):
-    """Return the argument which is True if only it is True.
+def exactly_one(*args):
+    """Return the argument which is True if only it is True among all arguments.
 
     Args:
-        _args: Arguments
+        args: Arguments
 
     Returns:
         The argument which evaluates to True.
 
     """
-    args = [*_args]
-    if any(args):
-        t = first(args, bool)
-        args.remove(t)
-        if all(map(lambda x: not x, args)):
-            return t
+    if sum(map(bool, args)) == 1:
+        return first(args, identity)
+    else:
+        return None
+
+
+def exactly_k(k, *args):
+    """Return all arguments which are :code:`True` if only if
+    exactly :code:`k` are :code:`True` among all arguments.
+
+    Args:
+        args: Arguments
+
+    Returns:
+        The arguments which evaluates to True.
+
+    """
+    if sum(map(bool, args)) == k:
+        return [*filter(identity, args)]
+    else:
+        return None
+
+
+
+# def cond(cases: Union[List[Tuple]]):
+#     pass
+
 
 # NOTE: Requires firstn
 # def at_most(k: int, *_args):
